searchData={"items":[{"type":"module","title":"RemoteIp","doc":"A plug to rewrite the `Plug.Conn`'s `remote_ip` based on forwarding headers.\n\nGeneric comma-separated headers like `X-Forwarded-For`, `X-Real-Ip`, and\n`X-Client-Ip` are all recognized, as well as the [RFC\n7239](https://tools.ietf.org/html/rfc7239) `Forwarded` header. IPs are\nprocessed last-to-first to prevent IP spoofing. Read more in the\ndocumentation for [the algorithm](algorithm.md).\n\nThis plug is highly configurable, giving you the power to adapt it to your\nparticular networking infrastructure:\n\n* IPs can come from any header(s) you want. You can even implement your own\n  custom parser if you're using a special format.\n\n* You can configure the IPs of known proxies & clients so that you never get\n  the wrong results.\n\n* All options are configurable at runtime, so you can deploy a single release\n  but still customize it using environment variables, the `Application`\n  environment, or any other arbitrary mechanism.\n\n* Still not getting the right IP? You can recompile the plug with debugging\n  enabled to generate logs, and even fine-tune the verbosity by selecting\n  which events to track.","ref":"RemoteIp.html"},{"type":"module","title":"Usage - RemoteIp","doc":"This plug should be early in your pipeline, or else the `remote_ip` might not\nget rewritten before your route's logic executes.\n\nIn [Phoenix](https://hexdocs.pm/phoenix), this might mean plugging `RemoteIp`\ninto your endpoint before the router:\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_app\n\n  plug RemoteIp\n  # plug ...\n  # plug ...\n  plug MyApp.Router\nend\n```\n\nBut if you only want to rewrite IPs in a narrower part of your app, you could\nof course put it in an individual pipeline of your router.\n\nIn an ordinary `Plug.Router`, you should make sure `RemoteIp` comes before\nthe `:match`/`:dispatch` plugs:\n\n```elixir\ndefmodule MyApp do\n  use Plug.Router\n\n  plug RemoteIp\n  plug :match\n  plug :dispatch\n\n  # get \"/\" do ...\nend\n```\n\nYou can also use `RemoteIp.from/2` to determine an IP from a list of headers.\nThis is useful outside of the plug pipeline, where you may not have access to\nthe `Plug.Conn`. For example, you might only be getting the `x_headers` from\n[`Phoenix.Socket`](https://hexdocs.pm/phoenix/Phoenix.Socket.html):\n\n```elixir\ndefmodule MySocket do\n  use Phoenix.Socket\n\n  def connect(params, socket, connect_info) do\n    ip = RemoteIp.from(connect_info[:x_headers])\n    # ...\n  end\nend\n```","ref":"RemoteIp.html#module-usage"},{"type":"module","title":"Configuration - RemoteIp","doc":"Options may be passed as a keyword list via `RemoteIp.init/1` or directly\ninto `RemoteIp.from/2`. At a high level, the following options are available:\n\n* `:headers` - a list of header names to consider\n* `:parsers` - a map from header names to custom parser modules\n* `:clients` - a list of known client IPs, either plain or in CIDR notation\n* `:proxies` - a list of known proxy IPs, either plain or in CIDR notation\n\nYou can specify any option using a tuple of `{module, function_name,\narguments}`, which will be called dynamically at runtime to get the\nequivalent value.\n\nFor more details about these options, see `RemoteIp.Options`.","ref":"RemoteIp.html#module-configuration"},{"type":"module","title":"Troubleshooting - RemoteIp","doc":"Getting the right configuration can be tricky. Requests might come in with\nunexpected headers, or maybe you didn't account for certain proxies, or any\nnumber of other issues.\n\nLuckily, you can debug `RemoteIp.call/2` and `RemoteIp.from/2` by updating\nyour `Config` file:\n\n```elixir\nconfig :remote_ip, debug: true\n```\n\nand recompiling the `:remote_ip` dependency:\n\n```console\n$ mix deps.clean --build remote_ip\n$ mix deps.compile\n```\n\nThen it will generate log messages showing how the IP gets computed. For more\ndetails about these messages, as well advanced usage, see\n`RemoteIp.Debugger`.","ref":"RemoteIp.html#module-troubleshooting"},{"type":"module","title":"Metadata - RemoteIp","doc":"When you use this plug, `RemoteIp.call/2` will populate the `Logger` metadata\nunder the key `:remote_ip`. This will be the string representation of the\nfinal value of the `Plug.Conn`'s `remote_ip`. Even if no client was found in\nthe headers, we still set the metadata to the original IP.\n\nYou can use this in your logs by updating your `Config` file:\n\n```elixir\nconfig :logger,\n  message: \"$metadata[$level] $message\\n\",\n  metadata: [:remote_ip]\n```\n\nThen your logs will look something like this:\n\n```log\n[info] Running ExampleWeb.Endpoint with cowboy 2.8.0 at 0.0.0.0:4000 (http)\n[info] Access ExampleWeb.Endpoint at http://localhost:4000\nremote_ip=1.2.3.4 [info] GET /\nremote_ip=1.2.3.4 [debug] Processing with ExampleWeb.PageController.index/2\n  Parameters: %{}\n  Pipelines: [:browser]\nremote_ip=1.2.3.4 [info] Sent 200 in 21ms\n```\n\nNote that metadata will *not* be set by `RemoteIp.from/2`.","ref":"RemoteIp.html#module-metadata"},{"type":"function","title":"RemoteIp.call/2","doc":"The `c:Plug.call/2` callback.\n\nRewrites the `Plug.Conn`'s `remote_ip` based on its forwarding headers. Each\ncall will re-evaluate all runtime options. See `RemoteIp.Options` for\ndetails.","ref":"RemoteIp.html#call/2"},{"type":"function","title":"RemoteIp.from/2","doc":"Extracts the remote IP from a list of headers.\n\nIn cases where you don't have access to a full `Plug.Conn` struct, you can\nuse this function to process the remote IP from a list of key-value pairs\nrepresenting the headers.\n\nYou may specify the same options as if you were using the plug. Runtime\noptions are evaluated each time you call this function. See\n`RemoteIp.Options` for details.\n\nIf no client IP can be found in the given headers, this function will return\n`nil`.","ref":"RemoteIp.html#from/2"},{"type":"function","title":"Examples - RemoteIp.from/2","doc":"iex> RemoteIp.from([{\"x-forwarded-for\", \"1.2.3.4\"}])\n    {1, 2, 3, 4}\n\n    iex> [{\"x-foo\", \"1.2.3.4\"}, {\"x-bar\", \"2.3.4.5\"}]\n    ...> |> RemoteIp.from(headers: ~w[x-foo])\n    {1, 2, 3, 4}\n\n    iex> [{\"x-foo\", \"1.2.3.4\"}, {\"x-bar\", \"2.3.4.5\"}]\n    ...> |> RemoteIp.from(headers: ~w[x-bar])\n    {2, 3, 4, 5}\n\n    iex> [{\"x-foo\", \"1.2.3.4\"}, {\"x-bar\", \"2.3.4.5\"}]\n    ...> |> RemoteIp.from(headers: ~w[x-baz])\n    nil","ref":"RemoteIp.html#from/2-examples"},{"type":"function","title":"RemoteIp.init/1","doc":"The `c:Plug.init/1` callback.\n\nThis accepts the keyword options described by `RemoteIp.Options`. Because\nplug initialization typically happens at compile time, we make sure not to\nevaluate runtime options until `call/2`.","ref":"RemoteIp.html#init/1"},{"type":"module","title":"RemoteIp.Debugger","doc":"Compile-time debugging facilities.\n\n`RemoteIp` uses the `debug/3` macro to instrument its implementation with\n*debug events* at compile time. If an event is enabled, the macro will expand\ninto a `Logger.debug/2` call with a specific message. If an event is\ndisabled, the logging will be purged, thus generating no extra code and\nhaving no impact on run time.","ref":"RemoteIp.Debugger.html"},{"type":"module","title":"Basic usage - RemoteIp.Debugger","doc":"Events are fired on every call to `RemoteIp.call/2` or `RemoteIp.from/2`. To\nenable or disable all debug events at once, you can set a boolean in your\n`Config` file:\n\n```elixir\nconfig :remote_ip, debug: true\n```\n\nBy default, the debugger is turned off (i.e., `debug: false`).\n\nBecause `RemoteIp.Debugger` works at compile time, you must make sure to\nrecompile the `:remote_ip` dependency whenever you change the configuration:\n\n```console\n$ mix deps.clean --build remote_ip\n```","ref":"RemoteIp.Debugger.html#module-basic-usage"},{"type":"module","title":"Advanced usage - RemoteIp.Debugger","doc":"You may also pass a list of atoms into the `:debug` configuration naming\nwhich events to log.\n\nThese are all the possible events:\n\n* `:options` - the keyword options *after* any runtime configuration has been\n  evaluated (see `RemoteIp.Options`)\n\n* `:headers` - all incoming headers, either from the `Plug.Conn`'s\n  `req_headers` or the list passed directly into `RemoteIp.from/2`; useful\n  for seeing if you're even getting the forwarding headers you expect in the\n  first place\n\n* `:forwarding` - the subset of headers (as configured by `RemoteIp.Options`)\n  that contain forwarding information\n\n* `:ips` - the entire sequence of IP addresses parsed from the forwarding\n  headers, in order\n\n* `:type` - for each IP (until we find the client), classifies the address\n  either as a known client, a known proxy, a reserved address, or none of the\n  above (and thus presumably a client)\n\n* `:ip` - the final result of the remote IP processing; when rewriting the\n  `Plug.Conn`'s `remote_ip`, the message will tell you the original IP that\n  is being replaced\n\nTherefore, `debug: true` is equivalent to passing in all of the above:\n\n```elixir\nconfig :remote_ip, debug: [:options, :headers, :forwarding, :ips, :type, :ip]\n```\n\nBut you could disable certain events by removing them from the list. For\nexample, to log only the incoming headers and resulting IP:\n\n```elixir\nconfig :remote_ip, debug: [:headers, :ip]\n```","ref":"RemoteIp.Debugger.html#module-advanced-usage"},{"type":"module","title":"Interactions with `Logger` - RemoteIp.Debugger","doc":"Since they both work at compile time, your configuration of `:logger` will\nalso affect the operation of `RemoteIp.Debugger`. For example, it's possible\nto enable debugging but still purge all the resulting logs:\n\n```elixir\n# All events *would* be logged...\nconfig :remote_ip, debug: true\n\n# ...But :debug logs will actually get purged at compile time\nconfig :logger, compile_time_purge_matching: [[level_lower_than: :info]]\n```","ref":"RemoteIp.Debugger.html#module-interactions-with-logger"},{"type":"macro","title":"RemoteIp.Debugger.debug/3","doc":"An internal macro for generating debug logs.\n\nThere is no reason for you to call this directly. It's used to instrument the\n`RemoteIp` module at compilation time.","ref":"RemoteIp.Debugger.html#debug/3"},{"type":"module","title":"RemoteIp.Headers","doc":"Functions for parsing IPs from multiple types of forwarding headers.","ref":"RemoteIp.Headers.html"},{"type":"function","title":"RemoteIp.Headers.parse/2","doc":"Parses IP addresses out of the given headers.\n\nFor each header name/value pair, the value is parsed for zero or more IP\naddresses by the parser corresponding to the name. If no such parser exists\nin the given map, we fall back to `RemoteIp.Parsers.Generic`.\n\nThe IPs are concatenated together into a single flat list. Note that the\nrelative order is preserved. That is, each header produce multiple IPs that\nare kept in the order given by that specific header. Then, in the case of\nmultiple headers, the concatenated list maintains the same order as the\nheaders appeared in the original name/value list.\n\nDue to the error-safe nature of the `RemoteIp.Parser` behaviour, headers that\ndo not actually contain valid IP addresses should be safely ignored.","ref":"RemoteIp.Headers.html#parse/2"},{"type":"function","title":"Examples - RemoteIp.Headers.parse/2","doc":"iex> [{\"x-one\", \"1.2.3.4, 2.3.4.5\"}, {\"x-two\", \"3.4.5.6, 4.5.6.7\"}]\n    ...> |> RemoteIp.Headers.parse()\n    [{1, 2, 3, 4}, {2, 3, 4, 5}, {3, 4, 5, 6}, {4, 5, 6, 7}]\n\n    iex> [{\"forwarded\", \"for=1.2.3.4\"}, {\"x-forwarded-for\", \"2.3.4.5\"}]\n    ...> |> RemoteIp.Headers.parse()\n    [{1, 2, 3, 4}, {2, 3, 4, 5}]\n\n    iex> [{\"accept\", \"*/*\"}, {\"user-agent\", \"ua\"}, {\"x-real-ip\", \"1.2.3.4\"}]\n    ...> |> RemoteIp.Headers.parse()\n    [{1, 2, 3, 4}]","ref":"RemoteIp.Headers.html#parse/2-examples"},{"type":"function","title":"RemoteIp.Headers.take/2","doc":"Extracts all headers with the given names.\n\nNote that `Plug.Conn` headers are assumed to have been normalized to\nlowercase, so the names you give should be in lowercase as well.","ref":"RemoteIp.Headers.html#take/2"},{"type":"function","title":"Examples - RemoteIp.Headers.take/2","doc":"iex> [{\"x-foo\", \"foo\"}, {\"x-bar\", \"bar\"}, {\"x-baz\", \"baz\"}]\n    ...> |> RemoteIp.Headers.take([\"x-foo\", \"x-baz\", \"x-qux\"])\n    [{\"x-foo\", \"foo\"}, {\"x-baz\", \"baz\"}]\n\n    iex> [{\"x-dup\", \"foo\"}, {\"x-dup\", \"bar\"}, {\"x-dup\", \"baz\"}]\n    ...> |> RemoteIp.Headers.take([\"x-dup\"])\n    [{\"x-dup\", \"foo\"}, {\"x-dup\", \"bar\"}, {\"x-dup\", \"baz\"}]","ref":"RemoteIp.Headers.html#take/2-examples"},{"type":"module","title":"RemoteIp.Options","doc":"The keyword options given to `RemoteIp.init/1` or `RemoteIp.from/2`.\n\nYou shouldn't need to use this module directly. Its functions are used\ninternally by `RemoteIp` to process configurations and support MFA-style\n[runtime options](#module-runtime-options).\n\nYou may pass any of the following keyword arguments into the plug (they get\npassed to `RemoteIp.init/1`). You can also pass the same keywords directly to\n`RemoteIp.from/2`.\n\n## `:headers`\n\nThe `:headers` option should be a list of strings. These are the names of\nheaders that contain forwarding information. The default is\n\n```elixir\n[\"forwarded\", \"x-forwarded-for\", \"x-client-ip\", \"x-real-ip\"]\n```\n\nEvery request header whose name exactly matches one of these strings will be\nparsed for IP addresses, which are then used to determine the routing\ninformation and ultimately the original client IP. Note that `Plug`\nnormalizes headers to lowercase, so this option should consist of lowercase\nnames.\n\nIn production, you likely want this to be a singleton - a list of only one\nstring. There are a couple reasons:\n\n1. You usually can't rely on servers to preserve the relative ordering of\n   headers in the HTTP request. For example, the\n   [Cowboy](https://github.com/ninenines/cowboy/) server presently [uses\n   maps](https://github.com/elixir-plug/plug_cowboy/blob/f82f2ff982f04fb4faa3a12fd2b08a7cc56ebe15/lib/plug/cowboy/conn.ex#L125-L127)\n   to represent headers, which don't preserve key order. The order in which\n   we process IPs matters because we take that as the routing information for\n   the request. So if you have multiple competing headers, the routing might\n   be ambiguous, and you could get bad results.\n\n2. It could also be a security issue. Say you're only expecting one header\n   like `X-Forwarded-For`, but configure multiple headers like\n   `[\"x-forwarded-for\", \"x-real-ip\"]`. Then it'd be easy for a malicious user\n   to just set an extra `X-Real-Ip` header and interfere with the IP parsing\n   (again, due to the sensitive nature of header ordering).\n\nWe still allow multiple headers because:\n\n1. Users can get up & running faster if the default configuration recognizes\n   all of the common headers.\n\n2. You shouldn't be relying that heavily on IP addresses for security. Even a\n   single plain-text header has enough problems on its own that we can't\n   guarantee its results are accurate. For more details, see the\n   documentation for [the algorithm](algorithm.md).\n\n3. It's more general. Networking setups are often very idiosyncratic, and we\n   want to give users the option to use multiple headers if that's what they\n   need.\n\n## `:parsers`\n\nThe `:parsers` option should be a map from strings to modules. Each string\nshould be a header name (lowercase), and each module should implement the\n`RemoteIp.Parser` behaviour. The default is\n\n\n```elixir\n%{\"forwarded\" => RemoteIp.Parsers.Forwarded}\n```\n\nHeaders with the given name are parsed using the given module. If a header is\nnot found in this map, it will be parsed by `RemoteIp.Parsers.Generic`. So\nyou can use this option to:\n\n* add a parser for your own custom header\n\n* specialize on the generic parsing of headers like `\"x-forwarded-for\"`\n\n* replace any of the default parsers with one of your own\n\nThe map you provide for this option is automatically merged into the default\nusing `Map.merge/2`. That way, the stock parsers won't be overridden unless\nyou explicitly provide your own replacement.\n\n## `:proxies`\n\nThe `:proxies` option should be a list of strings - either individual IPs or\nranges in\n[CIDR](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing)\nnotation. The default is\n\n\n```elixir\n[]\n```\n\nFor the sake of efficiency, you should prefer CIDR notation where possible.\nSo instead of listing out 256 different addresses for the `1.2.3.x` block,\nyou should say `\"1.2.3.0/24\"`.\n\nThese proxies are skipped by [the algorithm](algorithm.md) and are never\nconsidered the original client IP, unless specifically overruled by the\n`:clients` option.\n\nIn addition to the proxies listed here, note that the following [reserved IP\naddresses](https://en.wikipedia.org/wiki/Reserved_IP_addresses) are also\nskipped automatically, as they are presumed to be internal addresses that\ndon't belong to the client:\n\n* IPv4 loopback: `127.0.0.0/8`\n* IPv6 loopback: `::1/128`\n* IPv4 private network: `10.0.0.0/8`, `172.16.0.0/12`, `192.168.0.0/16`\n* IPv6 unique local address: `fc00::/7`\n\n## `:clients`\n\nThe `:clients` option should be a list of strings - either individual IPs or\nranges in\n[CIDR](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing)\nnotation. The default is\n\n\n```elixir\n[]\n```\n\nFor the sake of efficiency, you should prefer CIDR notation where possible.\nSo instead of listing out 256 different addresses for the `1.2.3.x` block,\nyou should say `\"1.2.3.0/24\"`.\n\nThese addresses are never considered to be proxies by [the\nalgorithm](algorithm.md). For example, if you configure the `:proxies` option\nto include `\"1.2.3.0/24\"` and the `:clients` option to include `\"1.2.3.4\"`,\nthen every IP in the `1.2.3.x` block would be considered a proxy *except* for\n`1.2.3.4`.\n\nThis option can also be used on reserved IP addresses that would otherwise be\nskipped automatically. For example, if your routing works through a local\nnetwork, you might actually consider addresses in the `10.x.x.x` block to be\nclients. You could permit the entire block with `\"10.0.0.0/8\"`, or even\nspecific IPs in this range like `\"10.1.2.3\"`.","ref":"RemoteIp.Options.html"},{"type":"module","title":"Runtime options - RemoteIp.Options","doc":"Every option can also accept a tuple of three elements: `{module, function,\narguments}` (MFA). These are passed to `Kernel.apply/3` at runtime, allowing\nyou to dynamically configure the plug, even though the `Plug.Builder`\ngenerally calls `c:Plug.init/1` at compilation time.\n\nThe return value from an MFA should be the same as if you were passing the\nliteral into that option. For instance, the `:proxies` MFA should return a\nlist of IP/CIDR strings.\n\nThe MFAs you give are re-evaluated on *each call* to `RemoteIp.call/2` or\n`RemoteIp.from/2`. So be careful not to do anything too expensive at runtime.\nFor example, don't download a list of known proxies, or else it will be\nre-downloaded on every request. Consider caching the download instead,\nperhaps using a library like [`Cachex`](https://hexdocs.pm/cachex).","ref":"RemoteIp.Options.html#module-runtime-options"},{"type":"module","title":"Examples - RemoteIp.Options","doc":"","ref":"RemoteIp.Options.html#module-examples"},{"type":"module","title":"Basic usage - RemoteIp.Options","doc":"Suppose you know:\n* you are behind proxies in the `1.2.x.x` block\n* the proxies use the `X-Real-Ip` header\n* but the IP `1.2.3.4` is actually a client, not one of the proxies\n\nThen you could say:\n\n```elixir\ndefmodule MyApp do\n  use Plug.Router\n\n  plug RemoteIp,\n    headers: ~w[x-real-ip],\n    proxies: ~w[1.2.0.0/16],\n    clients: ~w[1.2.3.4]\n\n  plug :match\n  plug :dispatch\n\n  # get \"/\" do ...\nend\n```\n\nThe same options may also be passed into `RemoteIp.from/2`:\n\n```elixir\ndefmodule MySocket do\n  use Phoenix.Socket\n\n  @options [\n    headers: ~w[x-real-ip],\n    proxies: ~w[1.2.0.0/16],\n    clients: ~w[1.2.3.4]\n  ]\n\n  def connect(params, socket, connect_info) do\n    ip = RemoteIp.from(connect_info[:x_headers], @options)\n    # ...\n  end\nend\n```","ref":"RemoteIp.Options.html#module-basic-usage"},{"type":"module","title":"Custom parser - RemoteIp.Options","doc":"Suppose your proxies are using a header with a special format. The name of\nthe header is `X-Special` and the format looks like `ip=127.0.0.1`.\n\nFirst, you'd implement a custom parser:\n\n```elixir\ndefmodule SpecialParser do\n  @behaviour RemoteIp.Parser\n\n  @impl RemoteIp.Parser\n  def parse(header) do\n    ip = String.replace_prefix(header, \"ip=\", \"\")\n    case :inet.parse_strict_address(ip |> to_charlist()) do\n      {:ok, parsed} -> [parsed]\n      _ -> []\n    end\n  end\nend\n```\n\nThen you would configure the plug with that parser. Make sure to also specify\nthe `:headers` option so that the `X-Special` header actually gets passed to\nthe parser.\n\n```elixir\ndefmodule SpecialApp do\n  use Plug.Router\n\n  plug RemoteIp,\n    headers: ~w[x-special],\n    parsers: %{\"x-special\" => SpecialParser}\n\n  plug :match\n  plug :dispatch\n\n  # get \"/\" do ...\nend\n```","ref":"RemoteIp.Options.html#module-custom-parser"},{"type":"module","title":"Using MFAs - RemoteIp.Options","doc":"Suppose you're deploying a release and you want to get the proxy IPs from an\nenvironment variable. Because the release is compiled ahead of time, you\nshouldn't do a `System.get_env/1` inline - it'll just be the value of the\nenvironment variable circa compilation time (probably empty!).\n\n```elixir\ndefmodule CompiledApp do\n  use Plug.Router\n\n  # DON'T DO THIS: the value of the env var gets compiled into the release\n  plug RemoteIp, proxies: System.get_env(\"PROXIES\") |> String.split(\",\")\n\n  plug :match\n  plug :dispatch\n\n  # get \"/\" do ...\nend\n```\n\nInstead, you can use an MFA to look up the variable at runtime:\n\n```elixir\ndefmodule RuntimeApp do\n  use Plug.Router\n\n  plug RemoteIp, proxies: {__MODULE__, :proxies, []}\n\n  def proxies do\n    System.get_env(\"PROXIES\") |> String.split(\",\", trim: true)\n  end\n\n  plug :match\n  plug :dispatch\n\n  # get \"/\" do ...\nend\n```","ref":"RemoteIp.Options.html#module-using-mfas"},{"type":"function","title":"RemoteIp.Options.default/1","doc":"The default value for the given option.","ref":"RemoteIp.Options.html#default/1"},{"type":"function","title":"RemoteIp.Options.pack/1","doc":"Processes keyword options, delaying the evaluation of MFAs until `unpack/1`.","ref":"RemoteIp.Options.html#pack/1"},{"type":"function","title":"RemoteIp.Options.unpack/1","doc":"Evaluates options processed by `pack/1`, applying MFAs as needed.","ref":"RemoteIp.Options.html#unpack/1"},{"type":"behaviour","title":"RemoteIp.Parser","doc":"Defines the interface for parsing headers into IP addresses.\n\n`RemoteIp.Headers.parse/1` dynamically dispatches to different parser modules\ndepending on the name of the header. For example, the `\"forwarded\"` header is\nparsed by `RemoteIp.Parsers.Forwarded`, which implements this behaviour.","ref":"RemoteIp.Parser.html"},{"type":"callback","title":"RemoteIp.Parser.parse/1","doc":"Parses the specific header's value into a list of IP addresses.\n\nThis callback should be error-safe. For instance, if the header's value is\ninvalid, it should return an empty list.\n\nThe actual work of converting an individual IP address string into the tuple\ntype should typically be done using `:inet` functions such as\n`:inet.parse_strict_address/1`.\n\nNote that a header may also contain more than one IP address. The order of\nthe list is important because it's interpreted as routing information.\nConceptually, the leftmost IP is the source of the request (the client), the\nrightmost IP is the destination (your server), and anything in the middle\nlists the proxy hops in order. However, in reality, there may be bad actors\nor strange routing that makes this more complicated. It's the job of\n`RemoteIp` to sort that out. This callback should *only* be concerned with\nfaithfully parsing the literal order given by the header.","ref":"RemoteIp.Parser.html#c:parse/1"},{"type":"type","title":"RemoteIp.Parser.t/0","doc":"Any module that implements the `RemoteIp.Parser` behaviour.","ref":"RemoteIp.Parser.html#t:t/0"},{"type":"module","title":"RemoteIp.Parsers.Forwarded","doc":"[RFC 7239](https://tools.ietf.org/html/rfc7239) compliant parser for\n`Forwarded` headers.\n\nThis module implements the `RemoteIp.Parser` behaviour. IPs are parsed out of\nthe `for=` pairs across each forwarded element.","ref":"RemoteIp.Parsers.Forwarded.html"},{"type":"module","title":"Examples - RemoteIp.Parsers.Forwarded","doc":"iex> RemoteIp.Parsers.Forwarded.parse(\"for=1.2.3.4;by=2.3.4.5\")\n    [{1, 2, 3, 4}]\n\n    iex> RemoteIp.Parsers.Forwarded.parse(\"for=\\\"[::1]\\\", for=\\\"[::2]\\\"\")\n    [{0, 0, 0, 0, 0, 0, 0, 1}, {0, 0, 0, 0, 0, 0, 0, 2}]\n\n    iex> RemoteIp.Parsers.Forwarded.parse(\"invalid\")\n    []","ref":"RemoteIp.Parsers.Forwarded.html#module-examples"},{"type":"module","title":"RemoteIp.Parsers.Generic","doc":"Generic parser for forwarding headers.\n\nThis module implements the `RemoteIp.Parser` behaviour. When there is not a\nmore specific parser, `RemoteIp.Headers.parse/1` falls back to using this\none.\n\nThe value is parsed simply as a comma-separated list of IPs. This is suitable\nfor a wide range of headers, such as `X-Forwarded-For`, `X-Real-IP`, and\n`X-Client-IP`.\n\nAny amount of whitespace is allowed before and after the commas, as well as\nat the beginning & end of the input.","ref":"RemoteIp.Parsers.Generic.html"},{"type":"module","title":"Examples - RemoteIp.Parsers.Generic","doc":"iex> RemoteIp.Parsers.Generic.parse(\"1.2.3.4, 5.6.7.8\")\n    [{1, 2, 3, 4}, {5, 6, 7, 8}]\n\n    iex> RemoteIp.Parsers.Generic.parse(\"  ::1  \")\n    [{0, 0, 0, 0, 0, 0, 0, 1}]\n\n    iex> RemoteIp.Parsers.Generic.parse(\"invalid\")\n    []","ref":"RemoteIp.Parsers.Generic.html#module-examples"},{"type":"extras","title":"Algorithm","doc":"# Algorithm\n\nTo avoid IP spoofing vulnerabilities, `RemoteIp` employs a very particular algorithm. Its work is divided into two main phases:\n\n1. Parse the right `req_headers`.\n2. Compute the right `remote_ip`.\n\nWe will analyze these steps in detail to understand the benefits and caveats of the algorithm. Much of it relies on configuration values given by `RemoteIp.Options`. You may also log the steps of this algorithm using the `RemoteIp.Debugger`.\n\nAs a running example, consider the following request route:\n\n* Client at IP `1.2.3.4` sends an HTTP request to Proxy 1 (no forwarding headers)\n* Proxy 1 at IP `1.1.1.1` adds an `X-Forwarded-For: 1.2.3.4` header and forwards to Proxy 2\n* Proxy 2 at IP `2.2.2.2` adds Proxy 1 to the header with `X-Forwarded-For: 1.2.3.4, 1.1.1.1` and forwards to Proxy 3\n* Proxy 3 at IP `3.3.3.3` adds a `Forwarded: for=2.2.2.2` header and forwards to the application\n* Application receives the request from IP `3.3.3.3` with forwarding headers `X-Forwarded-For: 1.2.3.4, 1.1.1.1` and `Forwarded: for=2.2.2.2`","ref":"algorithm.html"},{"type":"extras","title":"Parsing headers - Algorithm","doc":"There are many different forwarding headers in the wild, including `Forwarded`, `X-Forwarded-For`, `X-Client-Ip`, and `X-Real-Ip`. The header that gets used depends on the configuration of the proxy your app sits behind. If there are multiple proxies in play, it's conceivable for you to have more than one such header.\n\nThe `:headers` option tells `RemoteIp` which specific headers to parse for IP addresses. The default value casts a wide net, but you should ideally specify only those headers which you're certain you require. Otherwise, it would be trivial for a malicious client to add an extra header that could interfere with finding the correct IP.\n\nTo start off the algorithm, all of the configured headers are taken from the `Plug.Conn`'s `req_headers`. Their relative ordering is maintained, because order matters when there are multiple hops between proxies. In our running example, assuming both `Forwarded` and `X-Forwarded-For` were in the `:headers` option (which they are by default), we want to process the list\n\n```elixir\n# This is what we want\n[{\"x-forwarded-for\", \"1.2.3.4, 1.1.1.1\"}, {\"forwarded\", \"for=2.2.2.2\"}]\n```\n\nand *not* the reverse\n\n```elixir\n# This is NOT what we want\n[{\"forwarded\", \"for=2.2.2.2\"}, {\"x-forwarded-for\", \"1.2.3.4, 1.1.1.1\"}]\n```\n\nLet's assume we get the former. In reality, however, we usually can't rely on the stable ordering of headers in an HTTP request. For example, the [Cowboy](https://github.com/ninenines/cowboy/) server presently [uses maps](https://github.com/elixir-plug/plug_cowboy/blob/f82f2ff982f04fb4faa3a12fd2b08a7cc56ebe15/lib/plug/cowboy/conn.ex#L125-L127) to represent headers, which don't preserve key order, so everything could get jumbled up.\n\nConfiguring multiple headers might still be useful if, for example, you expect some requests to only have header A and other requests to only have header B, but never both at the same time. So `RemoteIp` doesn't limit you to just the one choice.\n\nAfter selecting the allowed headers, each string is parsed for its IP addresses. In the common case, we parse comma-separated IPs with `RemoteIp.Parsers.Generic`. This works for headers like `X-Forwarded-For`, `X-Client-Ip` and `X-Real-Ip`. But you can also configure custom parsers using the `:parsers` option. For instance, by default we include `RemoteIp.Parsers.Forwarded` to parse the format specified by RFC 7239.\n\nEach parser returns a list of IPs, each of the [`:inet.ip_address/0` type](http://erlang.org/doc/man/inet.html#type-ip_address). If there were any errors (e.g., a malformed header), this should be an empty list. But any one header may also specify multiple IPs, so once again it's important that the relative order is maintained. Thus, in our running example, the `X-Forwarded-For` header should parse as\n\n```elixir\n# This is what we want\n[{1, 2, 3, 4}, {1, 1, 1, 1}]\n```\n\nand *not* another order like\n\n```elixir\n# This is NOT what we want\n[{1, 1, 1, 1}, {1, 2, 3, 4}]\n```\n\nThe lists returned by each parser are then concatenated together to form one chain of IPs. In our running example, the resulting addresses are\n\n```elixir\n[{1, 2, 3, 4}, {1, 1, 1, 1}, {2, 2, 2, 2}]\n```","ref":"algorithm.html#parsing-headers"},{"type":"extras","title":"Finding the client - Algorithm","doc":"With the list of IPs parsed, `RemoteIp` must then calculate the proper `remote_ip`. To [prevent IP spoofing](http://blog.gingerlime.com/2012/rails-ip-spoofing-vulnerabilities-and-protection/), IPs are processed right-to-left. You can think of it as going backwards through the chain of hops:\n\n1. Application is receiving a request from Proxy 3\n2. The Proxy 3 to Application hop set the header `Forwarded: for=2.2.2.2`\n3. The Proxy 2 to Proxy 3 hop added `1.1.1.1` to the `X-Forwarded-For` header\n4. The Proxy 1 to Proxy 2 hop set the `X-Forwarded-For: 1.2.3.4` header\n5. Client is sending a request to Proxy 1\n\nWe work backwards until we find something that looks like a client IP. This is dictated by the `:proxies` option, which configures the list of known proxy IPs. Any IP that is *not* a known proxy is assumed to be a client. In our running example:\n\n1. The peer address `{3, 3, 3, 3}` is automatically assumed to be a proxy IP, so go through the headers\n2. `{2, 2, 2, 2}` is a known proxy IP, so go one hop back\n3. `{1, 1, 1, 1}` is a known proxy IP, so go one hop back\n4. `{1, 2, 3, 4}` is not a known proxy IP, so we assume it's the client\n\nNotice that the peer address is *always* assumed to be \"wrong\". Therefore, you should not use `RemoteIp` unless your app is behind at least one proxy. Otherwise, it would be trivial for a malicious client to spoof their IP address: if they just set a header themselves, we'll automatically use it to rewrite the `Plug.Conn`'s original (correct) peer address.\n\nIt's also important to go *backwards* through the chain, or else the client could similarly spoof their IP. For instance, consider if the client in the running example had initially sent the header `Forwarded: for=6.7.8.9`. Then the headers would have come in as\n\n```elixir\n[{\"forwarded\", \"for=6.7.8.9\"}, {\"x-forwarded-for\", \"1.2.3.4, 1.1.1.1\"}, {\"forwarded\", \"for=2.2.2.2\"}]\n```\n\nwhich would parse out as the IPs\n\n```elixir\n[{6, 7, 8, 9}, {1, 2, 3, 4}, {1, 1, 1, 1}, {2, 2, 2, 2}]\n```\n\nIf we were to go *forwards* through this list, we'd immediately return `{6, 7, 8, 9}` as the client IP, even though it was being spoofed by our malicious user. Instead, going backwards still gives us `{1, 2, 3, 4}` even though the client is attempting to spoof the IP with their own headers. This works no matter how many extra headers the client sends.\n\nThis logic generalizes to any bad actors in the middle of the chain, too. If we add an IP to the `:proxies` list, we're trusting the forwarding headers that it sets. As such, we're implicitly trusting the incoming peer address, even without configuring it. So in our running example, it's impossible not to trust Proxy 3. We believe it when it says the request came from Proxy 2. But if we didn't trust Proxy 2, that's where we stop: we say the client is `{2, 2, 2, 2}` and won't dig further because we don't trust the `X-Forwarded-For` header that came from Proxy 2.\n\nNot only are known proxies' headers trusted, but also requests forwarded for [loopback](https://en.wikipedia.org/wiki/Loopback) and [private](https://en.wikipedia.org/wiki/Private_network) IPs:\n\n* IPv4 loopback - `127.0.0.0/8`\n* IPv6 loopback - `::1/128`\n* IPv4 private network - `10.0.0.0/8`, `172.16.0.0/12`, `192.168.0.0/16`\n* IPv6 unique local address - `fc00::/7`\n\nThese IPs are skipped automatically because they are used internally and are thus generally not the actual client address in production. However, if (say) your app is only deployed in a [VPN](https://en.wikipedia.org/wiki/Virtual_private_network)/[LAN](https://en.wikipedia.org/wiki/Local_area_network), then your clients might actually have these internal IPs. To prevent loopback/private addresses from being considered proxies, configure them as known clients using the `:clients` option. This goes for anything you have listed in `:proxies` as well. For example, you might say that a whole CIDR block belongs to proxies, but then carve out an exception for a single client in that block.","ref":"algorithm.html#finding-the-client"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}